@@ -12,18 +12,16 @@ use std::path::Path;
 
 use crate::errors::Result;
 use crate::events::Event;
-use crate::name::{LocalName, NamespaceResolver, QName, ResolveResult};
-use crate::reader::{Reader, Span, XmlSource};
+use crate::name::{LocalName, NamespaceResolver, PrefixIter, QName, ResolveResult};
+use crate::reader::{Config, Reader, Span, XmlSource};
 
 /// A low level encoding-agnostic XML event reader that performs namespace resolution.
 ///
 /// Consumes a [`BufRead`] and streams XML `Event`s.
+#[derive(Debug, Clone)]
 pub struct NsReader<R> {
     /// An XML reader
     pub(super) reader: Reader<R>,
-    /// Buffer that contains names of namespace prefixes (the part between `xmlns:`
-    /// and an `=`) and namespace values.
-    buffer: Vec<u8>,
     /// A buffer to manage namespaces
     ns_resolver: NamespaceResolver,
     /// We cannot pop data from the namespace stack until returned `Empty` or `End`
@@ -40,7 +38,101 @@ impl<R> NsReader<R> {
         Self::new(Reader::from_reader(reader))
     }
 
-    configure_methods!(reader);
+    /// Returns reference to the parser configuration
+    #[inline]
+    pub const fn config(&self) -> &Config {
+        self.reader.config()
+    }
+
+    /// Returns mutable reference to the parser configuration
+    #[inline]
+    pub fn config_mut(&mut self) -> &mut Config {
+        self.reader.config_mut()
+    }
+
+    /// Returns all the prefixes currently declared except the default `xml` and `xmlns` namespaces.
+    ///
+    /// # Examples
+    ///
+    /// This example shows what results the returned iterator would return after
+    /// reading each event of a simple XML.
+    ///
+    /// ```
+    /// # use pretty_assertions::assert_eq;
+    /// use quick_xml::name::{Namespace, PrefixDeclaration};
+    /// use quick_xml::NsReader;
+    ///
+    /// let src = "<root>
+    ///   <a xmlns=\"a1\" xmlns:a=\"a2\">
+    ///     <b xmlns=\"b1\" xmlns:b=\"b2\">
+    ///       <c/>
+    ///     </b>
+    ///     <d/>
+    ///   </a>
+    /// </root>";
+    /// let mut reader = NsReader::from_str(src);
+    /// reader.config_mut().trim_text(true);
+    /// // No prefixes at the beginning
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![]);
+    ///
+    /// reader.read_resolved_event()?; // <root>
+    /// // No prefixes declared on root
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![]);
+    ///
+    /// reader.read_resolved_event()?; // <a>
+    /// // Two prefixes declared on "a"
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Default, Namespace(b"a1")),
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // <b>
+    /// // The default prefix got overridden and new "b" prefix
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2")),
+    ///     (PrefixDeclaration::Default, Namespace(b"b1")),
+    ///     (PrefixDeclaration::Named(b"b"), Namespace(b"b2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // <c/>
+    /// // Still the same
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2")),
+    ///     (PrefixDeclaration::Default, Namespace(b"b1")),
+    ///     (PrefixDeclaration::Named(b"b"), Namespace(b"b2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // </b>
+    /// // Still the same
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2")),
+    ///     (PrefixDeclaration::Default, Namespace(b"b1")),
+    ///     (PrefixDeclaration::Named(b"b"), Namespace(b"b2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // <d/>
+    /// // </b> got closed so back to the prefixes declared on <a>
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Default, Namespace(b"a1")),
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // </a>
+    /// // Still the same
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![
+    ///     (PrefixDeclaration::Default, Namespace(b"a1")),
+    ///     (PrefixDeclaration::Named(b"a"), Namespace(b"a2"))
+    /// ]);
+    ///
+    /// reader.read_resolved_event()?; // </root>
+    /// // <a> got closed
+    /// assert_eq!(reader.prefixes().collect::<Vec<_>>(), vec![]);
+    /// # quick_xml::Result::Ok(())
+    /// ```
+    #[inline]
+    pub const fn prefixes(&self) -> PrefixIter {
+        self.ns_resolver.iter()
+    }
 }
 
 /// Private methods
@@ -49,7 +141,6 @@ impl<R> NsReader<R> {
     fn new(reader: Reader<R>) -> Self {
         Self {
             reader,
-            buffer: Vec::new(),
             ns_resolver: NamespaceResolver::default(),
             pending_pop: false,
         }
@@ -66,7 +157,7 @@ impl<R> NsReader<R> {
 
     pub(super) fn pop(&mut self) {
         if self.pending_pop {
-            self.ns_resolver.pop(&mut self.buffer);
+            self.ns_resolver.pop();
             self.pending_pop = false;
         }
     }
@@ -74,11 +165,11 @@ impl<R> NsReader<R> {
     pub(super) fn process_event<'i>(&mut self, event: Result<Event<'i>>) -> Result<Event<'i>> {
         match event {
             Ok(Event::Start(e)) => {
-                self.ns_resolver.push(&e, &mut self.buffer);
+                self.ns_resolver.push(&e)?;
                 Ok(Event::Start(e))
             }
             Ok(Event::Empty(e)) => {
-                self.ns_resolver.push(&e, &mut self.buffer);
+                self.ns_resolver.push(&e)?;
                 // notify next `read_event_impl()` invocation that it needs to pop this
                 // namespace scope
                 self.pending_pop = true;
@@ -99,19 +190,9 @@ impl<R> NsReader<R> {
         event: Result<Event<'i>>,
     ) -> Result<(ResolveResult, Event<'i>)> {
         match event {
-            Ok(Event::Start(e)) => Ok((
-                self.ns_resolver.find(e.name(), &self.buffer),
-                Event::Start(e),
-            )),
-            Ok(Event::Empty(e)) => Ok((
-                self.ns_resolver.find(e.name(), &self.buffer),
-                Event::Empty(e),
-            )),
-            Ok(Event::End(e)) => Ok((
-                // Comment that prevent cargo rmt
-                self.ns_resolver.find(e.name(), &self.buffer),
-                Event::End(e),
-            )),
+            Ok(Event::Start(e)) => Ok((self.ns_resolver.find(e.name()), Event::Start(e))),
+            Ok(Event::Empty(e)) => Ok((self.ns_resolver.find(e.name()), Event::Empty(e))),
+            Ok(Event::End(e)) => Ok((self.ns_resolver.find(e.name()), Event::End(e))),
             Ok(e) => Ok((ResolveResult::Unbound, e)),
             Err(e) => Err(e),
         }
@@ -169,7 +250,7 @@ impl<R> NsReader<R> {
     /// [`resolve_element()`]: Self::resolve_element()
     #[inline]
     pub fn resolve<'n>(&self, name: QName<'n>, attribute: bool) -> (ResolveResult, LocalName<'n>) {
-        self.ns_resolver.resolve(name, &self.buffer, !attribute)
+        self.ns_resolver.resolve(name, !attribute)
     }
 
     /// Resolves a potentially qualified **element name** into _(namespace name, local name)_.
@@ -225,7 +306,7 @@ impl<R> NsReader<R> {
     /// [`read_resolved_event()`]: Self::read_resolved_event
     #[inline]
     pub fn resolve_element<'n>(&self, name: QName<'n>) -> (ResolveResult, LocalName<'n>) {
-        self.ns_resolver.resolve(name, &self.buffer, true)
+        self.ns_resolver.resolve(name, true)
     }
 
     /// Resolves a potentially qualified **attribute name** into _(namespace name, local name)_.
@@ -257,7 +338,6 @@ impl<R> NsReader<R> {
     /// ```
     /// # use pretty_assertions::assert_eq;
     /// use quick_xml::events::Event;
-    /// use quick_xml::events::attributes::Attribute;
     /// use quick_xml::name::{Namespace, QName, ResolveResult::*};
     /// use quick_xml::reader::NsReader;
     ///
@@ -267,7 +347,7 @@ impl<R> NsReader<R> {
     ///          xmlns='root namespace'
     ///          xmlns:p='other namespace'/>
     /// ");
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// match reader.read_event().unwrap() {
     ///     Event::Empty(e) => {
@@ -296,7 +376,7 @@ impl<R> NsReader<R> {
     /// [`Unknown`]: ResolveResult::Unknown
     #[inline]
     pub fn resolve_attribute<'n>(&self, name: QName<'n>) -> (ResolveResult, LocalName<'n>) {
-        self.ns_resolver.resolve(name, &self.buffer, false)
+        self.ns_resolver.resolve(name, false)
     }
 }
 
@@ -323,7 +403,7 @@ impl<R: BufRead> NsReader<R> {
     ///        <y:tag2>Test 2</y:tag2>
     ///     </x:tag1>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let mut count = 0;
     /// let mut buf = Vec::new();
@@ -340,7 +420,7 @@ impl<R: BufRead> NsReader<R> {
     ///             }
     ///         }
     ///         Event::Text(e) => {
-    ///             txt.push(e.unescape().unwrap().into_owned())
+    ///             txt.push(e.decode().unwrap().into_owned())
     ///         }
     ///         Event::Eof => break,
     ///         _ => (),
@@ -381,7 +461,7 @@ impl<R: BufRead> NsReader<R> {
     ///        <y:tag2>Test 2</y:tag2>
     ///     </x:tag1>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let mut count = 0;
     /// let mut buf = Vec::new();
@@ -399,7 +479,7 @@ impl<R: BufRead> NsReader<R> {
     ///         (_, Event::Start(_)) => unreachable!(),
     ///
     ///         (_, Event::Text(e)) => {
-    ///             txt.push(e.unescape().unwrap().into_owned())
+    ///             txt.push(e.decode().unwrap().into_owned())
     ///         }
     ///         (_, Event::Eof) => break,
     ///         _ => (),
@@ -434,7 +514,7 @@ impl<R: BufRead> NsReader<R> {
     /// Manages nested cases where parent and child elements have the _literally_
     /// same name.
     ///
-    /// If corresponding [`End`] event will not be found, the [`UnexpectedEof`]
+    /// If a corresponding [`End`] event is not found, an error of type [`IllFormed`]
     /// will be returned. In particularly, that error will be returned if you call
     /// this method without consuming the corresponding [`Start`] event first.
     ///
@@ -488,7 +568,7 @@ impl<R: BufRead> NsReader<R> {
     ///         </inner>
     ///     </outer>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     /// let mut buf = Vec::new();
     ///
     /// let ns = Namespace(b"namespace 1");
@@ -515,15 +595,15 @@ impl<R: BufRead> NsReader<R> {
     ///
     /// [`Start`]: Event::Start
     /// [`End`]: Event::End
-    /// [`UnexpectedEof`]: crate::errors::Error::UnexpectedEof
+    /// [`IllFormed`]: crate::errors::Error::IllFormed
     /// [`read_to_end()`]: Self::read_to_end
     /// [`BytesStart::to_end()`]: crate::events::BytesStart::to_end
-    /// [`expand_empty_elements`]: Self::expand_empty_elements
+    /// [`expand_empty_elements`]: Config::expand_empty_elements
     /// [the specification]: https://www.w3.org/TR/xml11/#dt-etag
     #[inline]
     pub fn read_to_end_into(&mut self, end: QName, buf: &mut Vec<u8>) -> Result<Span> {
         // According to the https://www.w3.org/TR/xml11/#dt-etag, end name should
-        // match literally the start name. See `Self::check_end_names` documentation
+        // match literally the start name. See `Config::check_end_names` documentation
         self.reader.read_to_end_into(end, buf)
     }
 }
@@ -569,7 +649,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///        <y:tag2>Test 2</y:tag2>
     ///     </x:tag1>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let mut count = 0;
     /// let mut txt = Vec::new();
@@ -585,7 +665,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///             }
     ///         }
     ///         Event::Text(e) => {
-    ///             txt.push(e.unescape().unwrap().into_owned())
+    ///             txt.push(e.decode().unwrap().into_owned())
     ///         }
     ///         Event::Eof => break,
     ///         _ => (),
@@ -630,7 +710,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///        <y:tag2>Test 2</y:tag2>
     ///     </x:tag1>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let mut count = 0;
     /// let mut txt = Vec::new();
@@ -647,7 +727,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///         (_, Event::Start(_)) => unreachable!(),
     ///
     ///         (_, Event::Text(e)) => {
-    ///             txt.push(e.unescape().unwrap().into_owned())
+    ///             txt.push(e.decode().unwrap().into_owned())
     ///         }
     ///         (_, Event::Eof) => break,
     ///         _ => (),
@@ -677,7 +757,7 @@ impl<'i> NsReader<&'i [u8]> {
     /// Manages nested cases where parent and child elements have the _literally_
     /// same name.
     ///
-    /// If corresponding [`End`] event will not be found, the [`UnexpectedEof`]
+    /// If a corresponding [`End`] event is not found, an error of type [`IllFormed`]
     /// will be returned. In particularly, that error will be returned if you call
     /// this method without consuming the corresponding [`Start`] event first.
     ///
@@ -726,7 +806,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///         </inner>
     ///     </outer>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let ns = Namespace(b"namespace 1");
     /// let start = BytesStart::from_content(r#"outer xmlns="namespace 1""#, 5);
@@ -752,14 +832,14 @@ impl<'i> NsReader<&'i [u8]> {
     ///
     /// [`Start`]: Event::Start
     /// [`End`]: Event::End
-    /// [`UnexpectedEof`]: crate::errors::Error::UnexpectedEof
+    /// [`IllFormed`]: crate::errors::Error::IllFormed
     /// [`BytesStart::to_end()`]: crate::events::BytesStart::to_end
-    /// [`expand_empty_elements`]: Self::expand_empty_elements
+    /// [`expand_empty_elements`]: Config::expand_empty_elements
     /// [the specification]: https://www.w3.org/TR/xml11/#dt-etag
     #[inline]
     pub fn read_to_end(&mut self, end: QName) -> Result<Span> {
         // According to the https://www.w3.org/TR/xml11/#dt-etag, end name should
-        // match literally the start name. See `Self::check_end_names` documentation
+        // match literally the start name. See `Config::check_end_names` documentation
         self.reader.read_to_end(end)
     }
 
@@ -800,7 +880,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///         <p>For example, elements not needed to be &quot;closed&quot;
     ///     </html>
     /// "#);
-    /// reader.trim_text(true);
+    /// reader.config_mut().trim_text(true);
     ///
     /// let start = BytesStart::new("html");
     /// let end   = start.to_end().into_owned();
@@ -808,7 +888,7 @@ impl<'i> NsReader<&'i [u8]> {
     /// // First, we read a start event...
     /// assert_eq!(reader.read_event().unwrap(), Event::Start(start));
     /// // ...and disable checking of end names because we expect HTML further...
-    /// reader.check_end_names(false);
+    /// reader.config_mut().check_end_names = false;
     ///
     /// // ...then, we could read text content until close tag.
     /// // This call will correctly handle nested <html> elements.
@@ -820,7 +900,7 @@ impl<'i> NsReader<&'i [u8]> {
     ///     "#));
     ///
     /// // Now we can enable checks again
-    /// reader.check_end_names(true);
+    /// reader.config_mut().check_end_names = true;
     ///
     /// // At the end we should get an Eof event, because we ate the whole XML
     /// assert_eq!(reader.read_event().unwrap(), Event::Eof);
